#ifndef _WIN32
#include <string.h>
#endif

#include <extdll.h>
#include <dllapi.h>
#include <h_export.h>
#include <meta_api.h>

#include "bot.h"
#include "bot_func.h"
#include "waypoint.h"
#include "bot_weapons.h"
#include "my_navigate.h"
#include "logging.h"

extern int mod_id;

/******* Something Different **********/
bool BotNeedBackPack(bot_t * pBot, TF_BackPack * pack)
{
	bool ammo_needed[4] = {FALSE, FALSE, FALSE, FALSE};
	int i;
	for (i = 0; i < 32; i++)
	{
		if (pBot->bot_weapons & (1<<i))
			if (pBot->weapons[i]->ShouldPickup(pack))
				return TRUE;
	}
	if (BotMaxHealth(pBot) < pBot->bot_health)
	{
		if (pack->health > 0)
		{
			return TRUE;
		}
	}
	if (BotMaxArmor(pBot) < pBot->bot_armor)
	{
		if (pack->armorvalue > 0)
		{
			return TRUE;
		}
	}

	return FALSE;
}

bool BotNeedAmmo(bot_t * pBot)
{
	for (int i = 0; i < 32; i++)
	{
		if (pBot->bot_weapons & (1<<i))
			if (pBot->weapons[i]->NeedAmmo())
				return TRUE;
	}
	return FALSE;
}

/****** CBotWeapon Functions ************/
CBotWeapon::CBotWeapon(bot_weapon_t bot_weapon, bot_ptr pBot)
{
	iAmmo1Index = bot_weapon.iAmmo1;
	iAmmo2Index = bot_weapon.iAmmo2;

	iAmmo1Max = bot_weapon.iAmmo1Max;
	iAmmo2Max = bot_weapon.iAmmo2Max;

	iClip = -1;

	iAmmo1Amt = 0;
	iAmmo2Amt = 0;
	iAmmo1Max = 0;
	iAmmo2Max = 0;

	iId = bot_weapon.iId;

	hasClip = FALSE;
	baseFitness = GetBaseFitnessFromFile(iId);
	underwater = TRUE;
	firingJitter = 1.0;
	firingArc = 15.0;
	this->pBot = pBot;
}

CBotWeapon::CBotWeapon(void)
{
	iId = -1;
	pBot=NULL;
	iAmmo1Index = -1;
	iAmmo2Index = -1;
}

float CBotWeapon::Fitness(Vector target, float targetDistance, TargetType targetType)
{
	return 0;
}

int CBotWeapon::Fire(Vector target)
{
	if (hasClip && iClip == 0)
	{
		pBot->pEdict->v.button |= IN_RELOAD;
		return BOT_WEAPON_RELOADING;
	}
	else
	{
		float toTurn = abs(pBot->pEdict->v.ideal_yaw - pBot->pEdict->v.v_angle.y);
		while (toTurn > 180.0)
		{
			toTurn -= 360.0;
		}
      while (toTurn < -180.0)
         toTurn += 360.0;
		if (toTurn < firingArc)
		{
			pBot->pEdict->v.button |= IN_ATTACK;
		}
	}
	return BOT_WEAPON_FIRED; //success
}

Vector CBotWeapon::GetBarrelOffset(void)
{
	if (pBot) return pBot->pEdict->v.view_ofs;
	else return Vector(0,0,0);
}

Vector CBotWeapon::GetTargetVector(Vector botOrigin, Vector targetOrigin, edict_t * pBotEnemy)
{
	return (targetOrigin-botOrigin);
}

Vector CBotWeapon::FuzzAiming(Vector aimVector)
{
	float f_distance = aimVector.Length();
	float f_scale, d_x, d_y, d_z, z_scale = 1.0, y_scale = 1.0;
	Vector target;
	int aimSkill;

    if (f_distance > 1000)
       f_scale = 1.0;
    else if (f_distance > 100)
       f_scale = f_distance / 1000.0;
    else
       f_scale = 0.1;

	if (false) //(pBot->fov != 0)
	{
		f_scale *= pBot->fov/90.0;
	}

	aimSkill = pBot->bot_skill -1;

   // randomly select the aiming style...
   target = aimVector;

   if (pBot->to_pitch > 20.0)
   {
	   z_scale = pBot->to_pitch/20.0;
   }

   if (pBot->to_yaw > 5.0)
   {
	   y_scale = pBot->to_yaw/5.0;
   }

   if (z_scale < 1.0)
	   z_scale = 1.0;
   if (y_scale < 1.0)
	   y_scale = 1.0;

   if (pBot->pEdict->v.button & IN_ATTACK)
   {
	   y_scale *= firingJitter;
	   z_scale *= firingJitter;
   }

   switch (aimSkill)
   {
      case 0:
         // VERY GOOD, same as from CBasePlayer::BodyTarget (in player.h)
         d_x = RANDOM_FLOAT(-2.5,2.5);
         d_y = RANDOM_FLOAT(-2.5,2.5);
         d_z = RANDOM_FLOAT(-5,5);
         break;
      case 1:
         // GOOD, offset a little for x, y, and z
         d_x = RANDOM_FLOAT(-5, 5) * f_scale;
         d_y = RANDOM_FLOAT(-5, 5) * f_scale;
         d_z = RANDOM_FLOAT(-10, 10) * f_scale;
         break;
      case 2:
         // FAIR, offset somewhat for x, y, and z
         d_x = RANDOM_FLOAT(-10, 10) * f_scale;
         d_y = RANDOM_FLOAT(-10, 10) * f_scale;
         d_z = RANDOM_FLOAT(-18, 18) * f_scale;
         break;
      case 3:
         // POOR, offset for x, y, and z
         d_x = RANDOM_FLOAT(-20, 20) * f_scale;
         d_y = RANDOM_FLOAT(-20, 20) * f_scale;
         d_z = RANDOM_FLOAT(-32, 32) * f_scale;
         break;
      case 4:
         // BAD, offset lots for x, y, and z
         d_x = RANDOM_FLOAT(-35, 35) * f_scale;
         d_y = RANDOM_FLOAT(-35, 35) * f_scale;
         d_z = RANDOM_FLOAT(-50, 50) * f_scale;
         break;
   }
   d_x *= y_scale;
   d_y *= y_scale;
   d_z *= z_scale;

   target = target + Vector(d_x, d_y, d_z);

   return target;

   float factor = 1.0;
	Vector answer = Vector(
		RANDOM_FLOAT(-100.0,100.0),
		RANDOM_FLOAT(-100.0,100.0),
		RANDOM_FLOAT(-100.0,100.0)
		);
	if (pBot->fov != 1)
	{
		factor = pBot->fov/90.0;
	}
	switch(pBot->bot_skill)
	{
	default:
	case 1:
		factor *= 1.0;
		break;
	case 2:
		factor *= 1.5;
		break;
	case 3:
		factor *= 2.0;
		break;
	case 4:
		factor *= 3.0;
		break;
	case 5:
		factor *= 5.0;
		break;
	}
	factor *= abs(aimVector.Length()/1000.0);
	return factor * answer;
}

bool CBotWeapon::CanFire()
{
	return (underwater) || (pBot->pEdict->v.waterlevel != 3);
}

bool CBotWeapon::ShouldPickup(TF_BackPack *pack)
{
	return FALSE;
}

bool CBotWeapon::NeedAmmo()
{
	if (this->iId > -1 && this->iAmmo1Max > -1)
		return (this->iAmmo1Amt < (this->iAmmo1Max/3));
	return FALSE;
}

/****** Melee Functions ***************/
CBotMelee::CBotMelee(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	hasClip = FALSE;
	clipSize = -1;
	baseFitness = GetBaseFitnessFromFile(bot_weapon.iId);
	if (baseFitness == -1.0)
	{
		baseFitness = 5.0;
	}
}

float CBotMelee::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness;
	if (!CanFire())
	{
		return 0.0;
	}
	if (distance > 100.0)
	{
		fitness *= 20.0/distance;
	}
	return fitness;
}

bool CBotMelee::ShouldPickup(TF_BackPack *pack)
{
	if (mod_id == TFC_DLL)
	{
		if (this->iId == TF_WEAPON_SPANNER)
		{
			if (this->iAmmo1Amt < 200)
			{
				if (pack->cells)
				{
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

bool CBotMelee::NeedAmmo()
{
	if (mod_id == TFC_DLL)
	{
		if (this->iId == TF_WEAPON_SPANNER)
		{
			if (this->iAmmo1Amt < 200)
			{
				return TRUE;
			}
		}
	}
	return FALSE;
}

/***** Pistol Functions **************/
CBotPistol::CBotPistol(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	baseFitness = GetBaseFitnessFromFile(bot_weapon.iId);
	if (baseFitness == -1.0)
	{
		baseFitness = 0.5;
	}

	if (mod_id == TFC_DLL)
	{
		hasClip = FALSE;
	} 
	else if (mod_id == FLF_DLL)
	{
		hasClip = TRUE;
		if (bot_weapon.iId == FLF_WEAPON_BERETTA)
		{
			clipSize = 16;
		}
		else //if (bot_weapon.iId == FLF_WEAPON_MK23)
		{
			clipSize = 12;
		}
	}
}

float CBotPistol::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (!CanFire())
	{
		return 0.0;
	}
	if (iAmmo1Amt > 0 || iClip > 9) //enough ammo?
	{
		//fitness *= 1.0;
	}
	else if (iClip > 0) //not much ammo
	{
		fitness *= 0.5;
	}
	else
	{
		return 0.0; //no ammo == no damage
	}
	
	if (distance > 100.0)
	{
		fitness *= 100.0/distance;
	}
	return fitness;
}

int CBotPistol::Fire(Vector target)
{
	if (hasClip && iClip == 0)
	{
		pBot->pEdict->v.button |= IN_RELOAD;
	}
	else
	{
		float toTurn = abs(pBot->pEdict->v.ideal_yaw - pBot->pEdict->v.v_angle.y);
		while (toTurn > 180.0)
		{
			toTurn -= 360.0;
		}
      while (toTurn < -180.0)
         toTurn += 360.0;
		if (toTurn < firingArc)
		{
			if (mod_id == FLF_DLL)
			{
				if (RANDOM_LONG(-20,10) > pBot->bot_skill)
				{
					pBot->pEdict->v.button |= IN_ATTACK;
				}
			}
			else //always shoot
			{
				pBot->pEdict->v.button |= IN_ATTACK;
			}
		}
	}
	return BOT_WEAPON_FIRED; //success
}

bool CBotPistol::ShouldPickup(TF_BackPack *pack)
{
	return FALSE;
}

/******** Shotgun Functions *****************/
CBotShotgun::CBotShotgun(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	baseFitness = GetBaseFitnessFromFile(bot_weapon.iId);
	if (mod_id == TFC_DLL)
	{
		hasClip = TRUE;
		if (bot_weapon.iId == TF_WEAPON_SHOTGUN)
		{
			clipSize = 8;
			iAmmo1Max = 40;
			if (baseFitness == -1.0)
				baseFitness = 1.4;
		} 
		else if (bot_weapon.iId == TF_WEAPON_SUPERSHOTGUN)
		{
			clipSize = 16;
			iAmmo1Max = 100;
			if (baseFitness == -1.0)
				baseFitness = 1.5;
		}
	}
	else if (mod_id == FLF_DLL)
	{
		hasClip = TRUE;
		if (bot_weapon.iId == FLF_WEAPON_SPAS12)
		{
			clipSize = 8;
		}
		else
		{
			clipSize = 10; //rsm2032
		}
		if (baseFitness == -1.0)
			baseFitness = 1.0;
	}
}

float CBotShotgun::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (iAmmo1Amt > 0 || iClip > 4) //enough ammo?
	{
		fitness *= 1.0;
	}
	else if (iClip > 0) //not much ammo
	{
		fitness *= 0.5;
	}
	else
	{
		return 0.0; //no ammo == no damage
	}
	
	if (distance > 200.0)
	{
		fitness *= 200.0/distance;
	}
	return fitness;
}

bool CBotShotgun::ShouldPickup(TF_BackPack *pack)
{
	if (this->iAmmo1Amt < this->iAmmo1Max)
	{
		if (pack->shells)
		{
			return TRUE;
		}
	}
	return FALSE;
}
