#ifndef _WIN32
#include <string.h>
#endif

#include <extdll.h>
#include <dllapi.h>
#include <h_export.h>
#include <meta_api.h>

#include "bot.h"
#include "bot_func.h"
#include "waypoint.h"
#include "bot_weapons.h"
#include "my_navigate.h"
#include "logging.h"

extern int mod_id;

/****** SMG Functions ***************/

CBotSMG::CBotSMG(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	if (mod_id == TFC_DLL)
	{
		hasClip = FALSE;
	}
	else if (mod_id == FLF_DLL)
	{
		hasClip = TRUE;
		if (bot_weapon.iId == FLF_WEAPON_UMP45)
		{
			clipSize = 25;
		}
		else
		{
			clipSize = 30;
		}
	}
	if (baseFitness == -1.0)
		baseFitness = 1.0;
}

float CBotSMG::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (iAmmo1Amt > 0 || iClip > 15) //enough ammo?
	{
		//fitness *= 1.0;
	}
	else if (iClip > 0) //not much ammo
	{
		fitness *= 0.5;
	}
	else
	{
		return 0.0; //no ammo == no damage
	}
	
	if (distance > 100.0)
	{
		fitness *= 100.0/distance;
	}
	return fitness;
}

/******* Rifle Functions ******************/

CBotRifle::CBotRifle(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	if (mod_id == FLF_DLL)
	{
		hasClip = TRUE;
		hasZoom = TRUE;
		if (bot_weapon.iId == FLF_WEAPON_SAKO)
		{
			clipSize = 6;
		} 
		else if (bot_weapon.iId == FLF_WEAPON_MSG90)
		{
			clipSize = 10;
		}
	}
	if (baseFitness == -1.0)
		baseFitness = 1.3;
}

float CBotRifle::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (pBot->state != STATE_SNIPING || pBot->stateTime < gpGlobals->time)
	{
		fitness /= 100.0; //shit when not sniping
	}
	if (iAmmo1Amt > 0 || iClip > 3) //enough ammo?
	{
		//fitness *= 1.0;
	}
	else if (iClip > 0) //not much ammo
	{
		fitness *= 0.5;
	}
	else
	{
		return 0.0; //no ammo == no damage
	}
	
	if (distance < 200)
	{
		fitness *= 0.5;
	}
	return fitness;
}

Vector CBotRifle::FuzzAiming(Vector aimVector)
{
	return aimVector;
}

int CBotRifle::Fire(Vector target)
{
	float distance = target.Length();

	if (pBot->fov == 0 || pBot->fov == 90.0)
	{
		if (distance > 100.0)
		{
			pBot->pEdict->v.button |= IN_ATTACK2;
		}
	}
	else
	{
		if (distance < 100.0)
		{
			pBot->pEdict->v.button |= IN_ATTACK2; //zoom out
		}
	}
	if (iClip == 0)
	{
		pBot->pEdict->v.button |= IN_RELOAD;
		return BOT_WEAPON_RELOADING;
	}
	pBot->pEdict->v.button |= IN_ATTACK;
	return BOT_WEAPON_FIRED;
}

Vector CBotRifle::GetTargetVector(Vector botOrigin, Vector targetOrigin, edict_t *pBotEnemy)
{
	Vector target = targetOrigin - botOrigin;
	if (pBot->bot_skill < 3)
	{
		target = (target + pBotEnemy->v.view_ofs + Vector(0,0,-25));
	}
	else
	{
		float z = RANDOM_FLOAT(-10,25);
		target = target + Vector(0,0,z);
	}
	return target;
}

/*********** Assault Rifle Functions ***************/
CBotAssault::CBotAssault(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	if (mod_id == FLF_DLL)
	{
		hasClip = TRUE;
		if (bot_weapon.iId == FLF_WEAPON_FAMAS)
		{
			clipSize = 25;
		}
		else if (bot_weapon.iId == FLF_WEAPON_AK5)
		{
			clipSize = 35;
		}
		else
		{
			clipSize = 30;
			hasZoom = TRUE;
		}
	}
	if (baseFitness == -1.0)
		baseFitness = 1.2;
}

float CBotAssault::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (iAmmo1Amt > 0 || iClip > 15) //enough ammo?
	{
		fitness *= 1.0;
	}
	else if (iClip > 0) //not much ammo
	{
		fitness *= 0.5;
	}
	else
	{
		return 0.0; //no ammo == no damage
	}
	
	if (distance > 300.0)
	{
		fitness *= 300.0/distance;
	}
	return fitness;
}
	

/*********** Heavy Weapons Functions *********************/
CBotHK21::CBotHK21(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	hasClip = TRUE;
	clipSize = 75;
	if (baseFitness == -1.0)
		baseFitness = 1.3;
}

float CBotHK21::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (iAmmo1Amt > 0 || iClip > 20) //enough ammo?
	{
		fitness *= 1.0;
	}
	else if (iClip > 0) //not much ammo
	{
		fitness *= 0.5;
	}
	else
	{
		return 0.0; //no ammo == no damage
	}
	
	if (distance > 300.0)
	{
		fitness *= 300.0/distance;
	}
	return fitness;
}

Vector CBotHK21::GetBarrelOffset()
{
	return Vector(0,0,0);
}

int CBotHK21::Fire(Vector target)
{
	if (iClip < 1)
	{
		//auto reload
		stopTime = 0;
		startTime = 0;
		return BOT_WEAPON_RELOADING;
	}
	if (stopTime < gpGlobals->time && startTime < gpGlobals->time)
	{
		startTime = gpGlobals->time;
		stopTime = gpGlobals->time+3.5; //burst time
	}
	if (stopTime < gpGlobals->time)
	{
		startTime = gpGlobals->time + 1.5;
		stopTime = gpGlobals->time + 5.0; //stop in 5
	}
	if (startTime > gpGlobals->time)
	{
		return BOT_WEAPON_RELOADING; //no shooting
	}
	if (gpGlobals->time - startTime > 1.0)
	{
		pBot->pEdict->v.v_angle.x -= 4.0; //correct aiming ?
	}
	pBot->pEdict->v.button |= IN_ATTACK;
	return BOT_WEAPON_FIRED;
}

/************** Grenade Functions *************************/
CBotGrenade::CBotGrenade(bot_weapon_t bot_weapon, bot_ptr pBot) :CBotWeapon(bot_weapon, pBot)
{
	hasClip = FALSE;
	clipSize = -1;
	if (baseFitness == -1.0)
		baseFitness = 1.0;
}

float CBotGrenade::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness;

	if (iAmmo1Amt < 1)
	{
		return 0.0; //no ammo
	}

	if (pBot->grenadeTime > gpGlobals->time) //not time yet :(
	{
		return 0.0;
	}

	if (distance > 800 || distance < 400)
	{
		return 0.01;
	}

	return fitness * RANDOM_LONG(0.5,1);
}

int CBotGrenade::Fire(Vector target)
{
	if (RANDOM_LONG(0, 100) < 50)
	{
		pBot->pEdict->v.button |=IN_ATTACK;
	}
	return BOT_WEAPON_FIRED;
}
