#ifndef _WIN32
#include <string.h>
#endif

#include <extdll.h>
#include <dllapi.h>
#include <h_export.h>
#include <meta_api.h>

#include "bot.h"
#include "my_navigate.h"
#include "waypoint.h"

bool NavIsFrontClear(edict_t *pEdict) {
	return NavIsDirectionReachable(pEdict, 0);
}

bool NavIsRearClear(edict_t *pEdict) {
	return NavIsDirectionReachable(pEdict, 180);
}

bool NavIsLeftClear(edict_t *pEdict) {
	return NavIsDirectionReachable(pEdict, 90);
}
bool NavIsRightClear(edict_t *pEdict) {
	return NavIsDirectionReachable(pEdict, -90);
}

bool NavIsDirectionReachable(edict_t *pEdict, float angle) {
	float checkDistance = 150; //check to see if we can get to
	Vector start = pEdict->v.origin;
	Vector direction = pEdict->v.v_angle;
	direction.y += angle;
	MAKE_VECTORS(direction);
	direction = gpGlobals->v_forward * checkDistance;
	direction = direction + start;
	return NavIsPointReachable(start, direction, pEdict);
}

bool NavIsDirectionReachable(edict_t *pEdict, float angle, Vector sight) {
	float checkDistance = 100; //check to see if we can get to
	Vector start = pEdict->v.origin;
	Vector direction = pEdict->v.v_angle;
	direction.y += angle;
	MAKE_VECTORS(direction);
	direction = gpGlobals->v_forward * checkDistance;
	direction = direction + start;
	if (NavIsPointReachable(start, direction, pEdict)) {
		TraceResult tr;
		UTIL_TraceLine(direction, sight, ignore_monsters, ignore_glass, pEdict, &tr);
		if (tr.flFraction >= 1.0) {
			return TRUE;
		}
	}
	return FALSE;
}

bool NavIsPointReachable(Vector v_start, Vector v_end, edict_t *pEdict) {
	//WaypointDrawBeam(edict_t *pEntity, Vector start, Vector end, int width,int noise, int red, int green, int blue, int brightness, int speed)
	//WaypointDrawBeam(INDEXENT(1), v_start, v_end, 30, 0, 255, 50, 50, 250, 5);
	return NavReachable(v_start, v_end, pEdict);
}

bool NavReachable(Vector v_src, Vector v_dest, edict_t *pEntity) {
	TraceResult tr;
	float curr_height, last_height;

	float distance = (v_dest - v_src).Length();

	if (distance < REACHABLE_RANGE) {
		// check if this waypoint is "visible"...

		UTIL_TraceLine( v_src, v_dest, ignore_monsters, pEntity->v.pContainingEntity, &tr );

		// if waypoint is visible from current position (even behind head)...
		if (tr.flFraction >= 1.0) {
			// check for special case of both waypoints being underwater...
			if ((POINT_CONTENTS( v_src ) == CONTENTS_WATER) && (POINT_CONTENTS( v_dest ) == CONTENTS_WATER)) {
				return TRUE;
			}

			// check for special case of waypoint being suspended in mid-air...
			// is dest waypoint higher than src? (45 is max jump height)
			if (v_dest.z > (v_src.z + 45.0)) {
				Vector v_new_src = v_dest;
				Vector v_new_dest = v_dest;

				v_new_dest.z = v_new_dest.z - 50;  // straight down 50 units

				UTIL_TraceLine(v_new_src, v_new_dest, dont_ignore_monsters, pEntity->v.pContainingEntity, &tr);

				// check if we didn't hit anything, if not then it's in mid-air
				if (tr.flFraction >= 1.0) {
					return FALSE;  // can't reach this one
				}
			}

			// check if distance to ground increases more than jump height
			// at points between source and destination...

			Vector v_direction = (v_dest - v_src).Normalize();  // 1 unit long
			Vector v_check = v_src;
			Vector v_down = v_src;

			v_down.z = v_down.z - 1000.0;  // straight down 1000 units

			UTIL_TraceLine(v_check, v_down, ignore_monsters, pEntity->v.pContainingEntity, &tr);

			last_height = tr.flFraction * 1000.0;  // height from ground

			distance = (v_dest - v_check).Length();  // distance from goal

			while (distance > 10.0) {
				// move 10 units closer to the goal...
				v_check = v_check + (v_direction * 10.0);

				v_down = v_check;
				v_down.z = v_down.z - 1000.0;  // straight down 1000 units

				UTIL_TraceLine(v_check, v_down, ignore_monsters, pEntity->v.pContainingEntity, &tr);

				curr_height = tr.flFraction * 1000.0;  // height from ground

				// is the difference in the last height and the current height
				// higher that the jump height?
        if ((last_height - curr_height) > 45.0 || (curr_height - last_height) > 100.0) {
					// can't get there from here...
					return FALSE;
				}

				last_height = curr_height;

				distance = (v_dest - v_check).Length();  // distance from goal
			}
			return TRUE;
		}
	}
	return FALSE;
}


bool NavIsClear(bot_t * pBot, TraceResult tr) {
	if (tr.flFraction >= 1.0) return TRUE;
	if (tr.pHit) {
		if (strcmp(STRING(tr.pHit->v.classname), "func_door") == 0) return TRUE;
		if (strcmp(STRING(tr.pHit->v.classname), "func_door_rotating") == 0) return TRUE;
	}
	return FALSE;
}
