#ifndef _WIN32
#include <string.h>
#endif

#include <extdll.h>
#include <dllapi.h>
#include <h_export.h>
#include <meta_api.h>

#include "bot.h"
#include "bot_func.h"
#include "waypoint.h"
#include "bot_weapons.h"
#include "my_navigate.h"
#include "logging.h"

extern int mod_id;

/********** Sniper Functions **********/
CBotSniperRifle::CBotSniperRifle(bot_weapon_t bot_weapon, bot_ptr pBot): CBotWeapon(bot_weapon, pBot)
{
	hasClip = FALSE;
	if (baseFitness == -1.0)
		baseFitness = 1.0;
	chargeTime = 0.0;
	reloadTime = 0.0;
	underwater = FALSE;
	firingJitter = 1.0;
	firingArc = 1.0;
};

float CBotSniperRifle::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness

	//less than 1/2 a clip?
	if (iAmmo1Amt == 0)
	{
		return 0.0;
	}
	else if (iAmmo1Amt < 3)
	{
		fitness *= 0.5;
	}

	if (distance < 200)
	{
		fitness *= 0.5;
	}
	return fitness;
}

int CBotSniperRifle::Fire(Vector target)
{
	float distance = target.Length();
	if (pBot->fov == 0)
	{
		if (distance > 100.0)
		{
			pBot->pEdict->v.button |= IN_ATTACK2;
		}
	}
	else
	{
		if (distance < 100.0)
		{
			pBot->pEdict->v.button |= IN_ATTACK2; //zoom out
		}
	}

	pBot->pEdict->v.button |= IN_ATTACK2;
	if (reloadTime > gpGlobals->time)
	{
		//do nothing
		return BOT_WEAPON_FIRED;
	}
	else if (chargeTime == 0.0)
	{
		float time;
		switch(pBot->bot_skill-1)
		{
		case 0:
		case 1: time = RANDOM_FLOAT(1.5,2.0); break;
		case 2: time = RANDOM_FLOAT(0.9,1.6); break;
		case 3: time = RANDOM_FLOAT(0.6,1.8); break;
		case 4: time = RANDOM_FLOAT(0.1,1); break;
		default: time = 0.0;
		}
		chargeTime = gpGlobals->time + time;
	}

	if (chargeTime > gpGlobals->time)
	{
		pBot->pEdict->v.button |= IN_ATTACK;
		return BOT_WEAPON_FIRED;
	}
	else //if (chargeTime < gpGlobals->time)
	{
		float toTurn = abs(pBot->pEdict->v.v_angle.y - pBot->pEdict->v.ideal_yaw);
		if (toTurn > 180.0)
		{
			toTurn -= 360.0;
		}
		if (toTurn < 1.0)
		{
			chargeTime = 0.0;
			reloadTime = gpGlobals->time + 2.0;
		}
		else
		{
			pBot->pEdict->v.button |= IN_ATTACK;
		}
		return BOT_WEAPON_FIRING;
	}
}

Vector CBotSniperRifle::GetTargetVector(Vector botOrigin, Vector targetOrigin, edict_t * pBotEnemy)
{
	Vector target = targetOrigin - botOrigin;
	if (pBot->bot_skill < 3)
	{
		if (chargeTime - gpGlobals->time > .15 && chargeTime != 0)
		{
			target = target + (2*pBotEnemy->v.view_ofs);
		}
		else
		{
			if (pBot->bot_skill == 2)
			{
				target = (target + pBotEnemy->v.view_ofs + Vector(0,0,-25));
			}
			else //skill == 0
			{
				switch(pBotEnemy->v.playerclass)
				{
				case TFC_CLASS_SCOUT:
				case TFC_CLASS_SNIPER:
				case TFC_CLASS_SPY:
					break; //aim for chest
				default:
					target = (target + pBotEnemy->v.view_ofs + Vector(0,0,-25));
				}
			}
		}
	}
	else
	{
		float z = RANDOM_FLOAT(-10,25);
		target = target + Vector(0,0,z);
	}
	return target;
}

bool CBotSniperRifle::ShouldPickup(TF_BackPack *pack)
{
	if (iAmmo1Amt < iAmmo1Max)
	{
		return (pack->shells != 0);
	}
	return FALSE;
}


/********* AutoRifle Functions ************/
CBotAutoRifle::CBotAutoRifle(bot_weapon_t bot_weapon, bot_ptr pBot): CBotWeapon(bot_weapon, pBot)
{
	if (baseFitness == -1.0)
		baseFitness = 0.3; //shithouse
	hasClip = FALSE;
}

float CBotAutoRifle::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness; //base fitness
	//less than 1/2 a clip?
	if (iAmmo1Amt == 0)
	{
		return 0.0;
	}
	else if (iAmmo1Amt < 15)
	{
		fitness *= 0.5;
	}


	if (distance < 200)
	{
		fitness *= 0.5;
	}
	return fitness;
}

bool CBotAutoRifle::ShouldPickup(TF_BackPack *pack)
{
	if (iAmmo1Amt < iAmmo1Max)
	{
		return (pack->shells != 0);
	}
	return FALSE;
}


/**************** Nailgun stuff **********************/
CBotNailgun::CBotNailgun(bot_weapon_t bot_weapon, bot_ptr pBot): CBotWeapon(bot_weapon, pBot)
{
	hasClip = FALSE;
	if (baseFitness == -1.0) {
		if (bot_weapon.iId == TF_WEAPON_NAILGUN)
		{
			baseFitness = 0.8;
		}
		else
		{
			baseFitness = 1.3;
		}
	}
	firingJitter = 2.0;
}

float CBotNailgun::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness;
	if (iAmmo1Amt == 0)
	{
		return 0.0;
	}
 	else if (iAmmo1Amt < 30)
	{
		fitness *= 0.5;
	}

	if (distance > 300.0)
	{
		fitness *= 300.0/distance;
	}
	return fitness;
}

bool CBotNailgun::ShouldPickup(TF_BackPack *pack)
{
	if (iAmmo1Amt < iAmmo1Max)
	{
		return (pack->nails != 0);
	}
	return FALSE;
}

/*************** GL Functions ******************/
/* max height: 240 */
/*
ideal firing angles for the weapon are:

arctan ( [2  +/-  sqrt ( 4 - x^2 - 4 y) )  /  x )

Where x and y are given in units of the maximum vertical range (the peak
if you shoot straight up), relative to your position.  If the square root
goes imaginary, then you're out of range
*/

CBotGLauncher::CBotGLauncher(bot_weapon_t bot_weapon, bot_ptr pBot): CBotWeapon(bot_weapon, pBot)
{
	hasClip = TRUE;
	clipSize = 6;
	if (baseFitness == -1.0)
		baseFitness = 1.5;
};

Vector CBotGLauncher::GetTargetVector(Vector botOrigin, Vector targetOrigin, edict_t * pBotEnemy)
{
	Vector toTarget = targetOrigin - botOrigin;
	Vector angles = UTIL_VecToAngles(toTarget);
	Vector2D horizontal = Vector2D(toTarget.z, toTarget.y);
	float x = horizontal.Length()/240.0;
	float x2 = x*x; //x = x^2
	float y = toTarget.z / 240.0;
	float root = (4- x2 - 4*y);
	if (root < 0)
	{
		return targetOrigin - botOrigin;
	}
	root = sqrt(root);
	root = 2+root;
	root /= x;
	float angle = atan(root);
	if (angle > 180)
	{
		angle -= 360.0;
	}
	angles.x = angle;
	MAKE_VECTORS(angles);
	return gpGlobals->v_forward * toTarget.Length();
}

bool CBotGLauncher::ShouldPickup(TF_BackPack *pack)
{
	if (this->iAmmo1Amt < this->iAmmo1Max)
	{
		return (pack->rockets != 0);
	}
	return FALSE;
}

float CBotGLauncher::Fitness(Vector toTarget, float distance, TargetType targetType)
{
	float fitness = baseFitness;
	if (iClip < 3 && iAmmo1Amt == 0)
	{
		fitness *= 0.5;
	}
	else if (iClip == 0 && iAmmo1Amt == 0)
	{
		return 0.0;
	}

	/* In range ? */
	Vector2D horizontal = Vector2D(toTarget.z, toTarget.y);
	float x = horizontal.Length()/240.0;
	x *= x; //x = x^2
	float y = toTarget.z / 240.0;
	if ((4-x - 4*y) < 0)
	{
		fitness *= 0.001; //no hit
	}


	/*if (toTarget.z > 50)
	{
		return 0.0; //can't shoot up with this thing
	}

	if (distance < 100.0)
	{
		fitness *= 0.01;
	}
	*/
	return fitness;
}

/*********** RPG and IC **************/
CBotRocket::CBotRocket(bot_weapon_t bot_weapon, bot_ptr pBot): CBotWeapon(bot_weapon, pBot)
{
	if (bot_weapon.iId == TF_WEAPON_RPG)
	{
		hasClip = TRUE;
		clipSize = 4;
		if (baseFitness == -1.0)
			baseFitness = 1.5;
	}
	else
	{
		hasClip = FALSE;
		if (baseFitness == -1.0)
			baseFitness = 1.0;
	}
	firingArc = 3.5;
}


float CBotRocket::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness;
	if (hasClip)
	{
		if (iAmmo1Amt == 0)
		{
			if (iClip == 0)
			{
				return 0.0;
			}
			fitness *= 0.5;
		}
	}
	else
	{
		if (iAmmo1Amt == 0)
		{
			return 0.0;
		}
		else if (iAmmo1Amt < 3)
		{
			fitness *= 0.5;
		}
	}
	if (distance < 300.0)
	{
		fitness *= 0.01;
	}
	else if (distance > 800.0)
	{
		fitness *= 800.0/distance;
	}
	if (targetType == TargetType_sentry)
	{
		if (this->iId == TF_WEAPON_RPG)
		{
			fitness *= 3.0; //great against sentris;
		}
		else //ic
		{
			fitness /= 3.0; //no great against sentries
		}
	}
	if (pBot->pBotEnemy)
	{
		float enemySpeed = pBot->pBotEnemy->v.velocity.Length();
		if (enemySpeed > 50.0)
		{
			fitness *= 100.0/enemySpeed;
		} else {
			fitness *= 2.0;
		}
	}
	return fitness;
}


Vector CBotRocket::GetTargetVector(Vector botOrigin, Vector targetOrigin, edict_t * pBotEnemy)
{
	Vector toTarget = targetOrigin - botOrigin;;
	if (pBot->bot_skill < 3)
	{
		TraceResult tr;
		//find the impact point for head:
		UTIL_TraceLine(botOrigin, botOrigin + 2* toTarget, ignore_monsters, pBot->pEdict, &tr);
		if (tr.flFraction < 1.0)
		{ //we have a hit
			if ((tr.vecEndPos - targetOrigin).Length() < 100.0)
			{
				return toTarget; //aim at head/chest area
			}
		}
		Vector feet = targetOrigin - (pBotEnemy->v.view_ofs);
		UTIL_TraceLine(botOrigin, feet, ignore_monsters, pBot->pEdict, &tr);
		if ((tr.flFraction >= 1.0) || (tr.vecEndPos - targetOrigin).Length() < 100.0)
		{ //hit feet or near them
			toTarget = feet - botOrigin;
		}
	}
	return toTarget;
}


bool CBotRocket::ShouldPickup(TF_BackPack *pack)
{
	if (iAmmo1Amt < iAmmo1Max)
	{
		return pack->rockets != 0;
	}
	return FALSE;
}

/************ IC Functions ***************************/
CBotFlameThrower::CBotFlameThrower(bot_weapon_t bot_weapon, bot_ptr pBot):CBotWeapon(bot_weapon, pBot)
{
	hasClip = FALSE;
	if (baseFitness == -1.0)
		baseFitness = 1.5;
	firingJitter = 1.5;
}

float CBotFlameThrower::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness;

	if (iAmmo1Amt == 0)
	{
		return 0.0;
	}
	else if (iAmmo1Amt < 20)
	{
		fitness *= 0.5;
	}
	if (distance > 500.0)
	{
		fitness *= 200.0/distance;
	}
	if (pBot->pBotEnemy)
	{
		if (pBot->pBotEnemy->v.playerclass == TFC_CLASS_PYRO)
		{
			return 0.0; // no good
		}
	}
	return fitness;
}

bool CBotFlameThrower::ShouldPickup(TF_BackPack *pack)
{
	if (iAmmo1Amt < iAmmo1Max)
	{
		return pack->cells != 0;
	}
	return FALSE;
}


/************ AC Functions ****************************/
CBotAutoCannon::CBotAutoCannon(bot_weapon_t bot_weapon, bot_ptr pBot):CBotWeapon(bot_weapon, pBot)
{
	hasClip = FALSE;
	if (baseFitness == -1.0)
		baseFitness = 1.5;
	//firingJitter = 2.0;
}

float CBotAutoCannon::Fitness(Vector target, float distance, TargetType targetType)
{
	float fitness = baseFitness;

	if (iAmmo1Amt == 0)
	{
		return 0.0;
	}
	else if (iAmmo1Amt < 20.0)
	{
		fitness *= 0.5;
	}

	if (distance > 300.0)
	{
		fitness *= 300.0/distance;
	}
	return fitness;
}

Vector CBotAutoCannon::GetBarrelOffset()
{
	return Vector(0,0,0);
}

bool CBotAutoCannon::ShouldPickup(TF_BackPack *pack)
{
	if (iAmmo1Amt < iAmmo1Max)
	{
		return pack->shells != 0;
	}
	return FALSE;
}

/*

//#define TF_WEAPON_IC            15
class CBotIC:public CBotWeapon
{
public:
	CBotIC(bot_weapon_t);
	float Fitness(bot_ptr pBot, Vector toTarget);
};

//#define TF_WEAPON_AC            17
class CBotAutoCannon: public CBotWeapon
{
public:
	CBotAutoCannon(bot_weapon_t);
	float Fitness(bot_ptr pBot, Vector toTarget);
};

*/