#ifndef _WIN32
#include <string.h>
#endif

#include <extdll.h>
#include <dllapi.h>
#include <h_export.h>
#include <meta_api.h>

#include "bot.h"
#include "fuzzy.h"

#define AT 0
#define CLOSE 1
#define MEDIUM 2
#define FARISH 3

//the max number of distinct ranges
#define RANGES 4

#define BFAST (0)
#define BMID (1)
#define BSLOW (2)
#define STOPPED (3)
#define SLOW (4)
#define MID (5)
#define FAST (6)

#define SPEEDS (7)

#define MAXDIST (1000)
#define MAXHEIGHT (100)
#define MINDIST (0)

typedef struct {
	float min, mid, max;
} triangle_t;

int matrix[RANGES][RANGES] = {
	{STOPPED, SLOW, MID, FAST},
	{BSLOW, STOPPED, SLOW, MID},
	{BMID, BSLOW, STOPPED, SLOW},
	{BFAST, BMID, BSLOW, STOPPED}
};

triangle_t distances[RANGES] = {
	{-300,0,300},
	{0,200,400},
	{100,400,700},
	{400,1000,1600}
};

triangle_t speeds[SPEEDS] = {
	{-1.3, -1, -0.7},
	{-0.8, -0.5, -0.2},
	{-0.3, -0.1, 0.1},
	{0.2, 0 , 0.2},
	{-0.1, 0.1, 0.3},
	{0.2, 0.5, 0.8},
	{0.7, 1, 1.3}
};

float Triangle(triangle_t tri, float x);


float Triangle(triangle_t tri, float x) {
	float m,c;
	
	//out of bounds
	if (x < tri.min || x > tri.max) return 0;
	
	if (x < tri.mid) {
		m = MAXHEIGHT/(tri.mid-tri.min);
		c = -m * (tri.min);
	} else if (x > tri.mid) {
		m = MAXHEIGHT/(tri.mid-tri.max);
		c = -m * tri.max;
	} else {
		return MAXHEIGHT; //right on the apex = max height
	}
	return (m * x + c);
}

float GetSpeedRatio(float dist, float pref) {
	float membership[2][RANGES];
	float answers[SPEEDS];
	float totalArea = 0;
	float totalX = 0;	
	int i=0,j;
	
	for (i = 0; i < SPEEDS; i++) {
		answers[i] = 0; //zero the array
	}
	
	dist = MIN(dist, MAXDIST);
	pref = MIN(pref, MAXDIST);
	
	for (j = 0; j < RANGES; j++) {
		membership[0][j] = Triangle(distances[j], dist);
		membership[1][j] = Triangle(distances[j], pref);
	}
	
	
	for(i = 0; i < RANGES; i++) {
		for (j = 0; j < RANGES; j++) {
			answers[matrix[i][j]] += MIN(membership[0][j], membership[1][i]);
		}
	}
	
	//find the centroid
	for (i = 0; i < SPEEDS; i++) {
		if (answers[i] == 0) continue;
		float area = (0.5) * (speeds[i].max-speeds[i].min) * answers[i];
		totalArea += area;
		totalX += (speeds[i].mid * area);
	}
		
	if (totalArea == 0) {
		//cog = 0;
		return 0;
	} else {
		float cog = totalX / totalArea;
		if (cog < -1) {
			return -1;
		} else if (cog > 1) {
			return 1;
		} else {
			return cog;
		}
	}		
}
